searchData={"items":[{"type":"module","title":"JungleSpec","doc":"A module providing simplified and less verbose way of definining OpenApiSpex types.\n\nJungleSpec provides two main entry points: `open_api_object` to define an object, and `open_api_type` to define\na non-object type. Both generate an OpenApiSpec schema (`OpenApiSpex.Schema`) and an internal Elixir type.\n\nOnly a single `open_api_object` or `open_api_type` clause is allowed per module, as the underlying Elixir type\nis defined as the `t()`-type within that module.\n\nThe generated OpenApiSpex schema can be obtained by calling `schema/1` function on the module.\n\nThe OpenAPI specification requires that each schemas has a unique name, called title. This is provided as the\nfirst argument for both `open_api_object` and `open_api_type` and must be a string.\n\n# Non-object Schemas\n`open_api_type` is used to define non-object schemas. That could be enumerable strings, regexp-limited strings,\nunion types, and similar.\n\nExample:\n\n    defmodule EnumExample do\n      use JungleSpec\n\n      open_api_type \"NonObjectExample\", :string, enum: [\"ValueA\", \"ValueB\"]\n    end\n\nThe above example will create the following OpenApiSpex schema struct:\n\n    %OpenApiSpex.Schema%{\n      type: :string,\n      enum: [\"ValueA\", \"ValueB\"],\n      title: \"NonObjectExample\",\n      nullable: false\n    }\n\nElixir-wise, the `t` type is defined as\n\n    @type t() :: String.t()\n\n\nSee `open_api_type/3` for more details in supported types and options.\n\n# Object Schemas\n`open_api_object/3` is used to specify object-type schemas. By default, object schemas are mapped to Elixir structs,\nbut can be mapped to maps instead (see the \"Elixir Struct or Map\"-section for the details).\n\n`open_api_object` is used to set the title of the object-schema, and object-level options.\nThe properties of the object are defined using the `property/3` macro. Here two parameters are required: The\nname and type of the property.\n\nExample:\n\n    defmodule Person do\n      use JungleSpec\n\n      open_api_object \"Person\" do\n        property :name, :string\n        property :age, :integer\n      end\n    end\n\nThis will generate the following OpenApiSpex Schema struct:\n\n    %OpenApiSpex.Schema%{\n      type: :object,\n      title: \"Person\",\n      required: [:name, :age],\n      nullable: false,\n      properties: %{\n        name: %OpenApiSpex.Schema%{type: :string, nullable: false},\n        age: %OpenApiSpex.Schema%{type: :integer, nullable: false}\n      },\n      \"x-struct\": Person\n    }\n\nAnd the following Elixir type (struct):\n\n    @type t() :: %Person{\n              age: integer(),\n              name: String.t()\n            }","ref":"JungleSpec.html"},{"type":"module","title":"Required and Nullable Properties - JungleSpec","doc":"By default, all properties are \"required\". That means that the generated OpenApiSpec schema will have all\nproperties in the list of `required` properties.\n\nIt is possible to set `required: false` for a specific property. However, then a `default` value MUST be provided,\nor a compile-time error is emitted:\n\n    defmodule Person do\n      use JungleSpec\n\n      open_api_object \"Person\" do\n        property :name, :string\n        property :age, :integer, required: false, default: 0\n      end\n    end\n\nPlease note, that even when `required: false` for a field, the Elixir struct still needs the value provided!\n\nIt is also possible to set `required: false` on the entire object, and only setting `required: true` on some fields:\n\n    defmodule Person do\n      use JungleSpec\n\n      open_api_object \"Person\", required: false do\n        property :name, :string, required: true\n        property :age, :integer, default: 0\n      end\n    end\n\nA default value must still be provided for `:age` in order for this to compile.\n\nAll properties have a default setting of `nullable: false`, i.e. they cannot be `nil`/`null`.\nBy setting `nullable: true` on a property, `nil`/`null` becomes a valid value. This will also change the Elixir-struct,\nsuch that the property is no longer required (but will default to `nil` if not given) -- even when a default value other\nthan `nil` is provided.","ref":"JungleSpec.html#module-required-and-nullable-properties"},{"type":"module","title":"Elixir Struct or Map - JungleSpec","doc":"Elixir structs have the property, that all declared fields are always present. If it is necessary to define an\nobject, where not all properties are present all the time, we cannot map that into an Elixir struct. Instead we\ncan map it to an Elixir map, by providing the `struct?: false` option to `open_api_object`:\n\n    defmodule Person do\n      use JungleSpec\n\n      open_api_object \"Person\", struct?: false do\n        property :name, :string\n        property :age, :integer, required: false\n      end\n    end\n\nNote, that it is no longer necessary to provide a default value for `:age`, as the field is allowed not to be\npresent. The Elixir type for this schema is:\n\n    @type t() :: %{\n              age: integer(),\n              name: String.t()\n            }\n\nTwo important things to note here: First, it is a map, an not a struct. Second, it wrongly, says that `:age` is required.\nThe typespec should have had an `optional(age) => integer()`, but this has not yet been implemented.\nIn practice, this is only an issue if the type is used, and Dialyzer catches the bug. The main important point of doing\nthis, is that the generated OpenApiSpex schema struct instructs OpenApiSpex to generate a map and not a struct.\nNot optimal, but it works :-)","ref":"JungleSpec.html#module-elixir-struct-or-map"},{"type":"module","title":"Schema Extension (Inheritance) - JungleSpec","doc":"Sometimes, it can be helpful to create a schema that is an extension of another schema.\n\nThe following schema extends the `Person` schema used as a previous example:\n\n    defmodule Employee do\n      use JungleSpec\n\n      open_api_object \"Employee\", extends: Person do\n        property :level, :string, enum: [\"L1\", \"L2\", \"L3\"]\n        property :experience, [:number, :string]\n      end\n    end\n\n  This will simply copy over all properties from `Person` and include them along with the the new ones provided.\n  The OpenApiSpex schema structure would look like:\n\n    %OpenApiSpex.Schema%{\n      type: :object,\n      title: \"Employee\",\n      required: [:name, :age, :level, :experience],\n      nullable: false,\n      properties: %{\n        name: %OpenApiSpex.Schema%{type: :string, nullable: false},\n        level: %OpenApiSpex.Schema%{\n          type: :string,\n          enum: [\"L1\", \"L2\", \"L3\"],\n          nullable: false\n        },\n        experience: %OpenApiSpex.Schema%{\n          oneOf: [\n            %OpenApiSpex.Schema%{type: :number, nullable: false},\n            %OpenApiSpex.Schema%{type: :string, nullable: false}\n          ]\n        },\n        age: %OpenApiSpex.Schema%{type: :integer, nullable: false}\n      },\n      \"x-struct\": Employee\n    }\n\n  While the Elixir type will be:\n\n    @type t() :: %Employee{\n        age: integer(),\n        experience: number() | String.t(),\n        level: String.t(),\n        name: String.t()\n      }","ref":"JungleSpec.html#module-schema-extension-inheritance"},{"type":"macro","title":"JungleSpec.additional_properties/2","doc":"Macro for defining object's `additionalProperties`. It has one required argument:\n\n  * `type` - type have to be one of the allowed types defined in the `property` macro\n\nIt has also one option:\n\n  * `:nullable` - a boolean value specifying if the additional properties can be nullable.\n  `false` by default","ref":"JungleSpec.html#additional_properties/2"},{"type":"function","title":"JungleSpec.define_property/4","doc":"","ref":"JungleSpec.html#define_property/4"},{"type":"function","title":"JungleSpec.enforced_properties/1","doc":"","ref":"JungleSpec.html#enforced_properties/1"},{"type":"macro","title":"JungleSpec.maybe_define_struct/2","doc":"","ref":"JungleSpec.html#maybe_define_struct/2"},{"type":"macro","title":"JungleSpec.open_api_object/1","doc":"","ref":"JungleSpec.html#open_api_object/1"},{"type":"macro","title":"JungleSpec.open_api_object/2","doc":"","ref":"JungleSpec.html#open_api_object/2"},{"type":"macro","title":"JungleSpec.open_api_object/3","doc":"Defines OpenApiSpex schema with `:object` type and appropriate typespec.\n\nIt has one required argument:\n\n  * `title` - a binary being a title of the schema\n\nIt is followed by an optional keyword list of options and an optional `do ... end` block\ncontaining object's properties.\n\nPossible options\n\n  * `:description` - a binary describing the whole object\n\n  * `:example` - a map being an example of the object defined by the schema\n\n  * `:extends` - a module of the schema that this schema extends. All properties of the extended\n    schema will be added to the current schema, preserving the information if the properties are\n    required. It does not propagate nullability of the extended schema. Ensure that\n    `open_api_object/3` macro was used to define the extended module\n\n  * `:inline` - a boolean value specifying the default `inline` option for all properties. By\n    default it is `false`\n\n  * `:nullable` - a boolean value specifying whether the object can be nullable. `false` by\n    default\n\n  * `:required` - a boolean value specifying the default `required` option for all properties. By\n    default it is `true`\n\n  * `:struct?` - a boolean value telling OpenApiSpex if it should create a struct out of the\n    object. It will also add the struct to the typespec if set to `true`. By default it is `true`","ref":"JungleSpec.html#open_api_object/3"},{"type":"macro","title":"JungleSpec.open_api_type/3","doc":"Defines OpenApiSpex schema with type different from `:object` and appropriate typespec.\n\nIt has two required arguments:\n\n  * `title` - a binary being a title of the schema\n\n  * `type` - an atom proving the type of the schema.\n\nIt is followed by an optional keyword list of options.\n\nAllowed types:\n\n  * `:integer` - maps to the type of the same name.\n\n  * `:number` - either integer or float. Maps to the type of the same name.\n\n  * `:string` - a binary. Maps to the type of the same name.\n\n  * `:boolean` - maps to the type of the same name.\n\n  * `{:array, type}` - a list of elements having provided type. `type` have to be one of the\n    allowed types.\n\n  * `{:map, type}` - a nested object with additional properties having provided `type`, which\n    have to be one of the allowed types.\n\n  * `[type_1, type_2, ...]` - a union of the provided types. The types have to be allowed.\n\n  * `module` - a module name that has it's own schema.\n\nSupported options are:\n\n  * `:default` - default value for the property. It has to match type of the property.\n\n  * `:description` - a binary describing the property.\n\n  * `:enum` - a list of possible values which have to be binaries. Only valid for `:string` type.\n\n  * `:example` - an example of the property. It has to match its type.\n\n  * `:format` - an atom describing property's format.\n\n  * `:inline` - a boolean value that is used if the property's type is another module. Then,\n    if `inline: true`, the module's schema is just inlined. Otherwise, only the reference is\n    used. By default, the value is propagated from the object.\n\n  * `:nullable` - a boolean value specifying if the property can be `nil`. `false` by default.\n\n  * `:pattern` - a regular expression describing possible format of the property. Only valid for `:string` type.","ref":"JungleSpec.html#open_api_type/3"},{"type":"function","title":"JungleSpec.prepare_object_schema/5","doc":"","ref":"JungleSpec.html#prepare_object_schema/5"},{"type":"function","title":"JungleSpec.prepare_type_schema/4","doc":"","ref":"JungleSpec.html#prepare_type_schema/4"},{"type":"macro","title":"JungleSpec.property/3","doc":"Defines one property for the open_api_object.\n\nIt has two required arguments:\n\n  * `name` - an atom being a name of the property\n\n  * `type` - an atom or a tuple proving the type of the schema\n\nAllowed types:\n\n  * `:integer` - maps to the type of the same name\n\n  * `:number` - either integer or float. Maps to the type of the same name\n\n  * `:string` - a binary. Maps to the type of the same name\n\n  * `:boolean` - maps to the type of the same name\n\n  * `{:array, type}` - a list of elements having provided type. `type` have to be one of the\n    allowed types\n\n  * `{:map, type}` - a nested object with additional properties having provided `type`, which\n    have to be one of the allowed types\n\n  * `[type_1, type_2, ...]` - a union of the provided types. The types have to be allowed\n\n  * `module` - a module name that has it's own schema\n\nProperty also has an optional keyword list with the following possible options:\n\n  * `:default` - default value for the property. It has to match type of the property\n\n  * `:description` - a binary describing the property\n\n  * `:enum` - a list of possible values which have to be binaries. Also, property has to have\n    `:string` type\n\n  * `:example` - an example of the property. It has to match its type\n\n  * `:format` - an atom describing property's format\n\n  * `:inline` - a boolean value that is used if the property's type is another module. Then,\n    if `inline: true`, the module's schema is just inlined. Otherwise, only the reference is\n    used. By default, the value is propagated from the object\n\n  * `:nullable` - a boolean value specifying if the property can be `nil`. `false` by default\n\n  * `:pattern` - a regular expression describing possible format of the property\n\n  * `:required` - a boolean value specifying if the property should be added to the list of\n    object's required properties. By default, it is propagated from the object","ref":"JungleSpec.html#property/3"},{"type":"function","title":"JungleSpec.struct_definition/1","doc":"","ref":"JungleSpec.html#struct_definition/1"},{"type":"module","title":"JungleSpec.TypespecGenerator","doc":"Generator of typespecs for OpenApiSpex.Schema structs.","ref":"JungleSpec.TypespecGenerator.html"},{"type":"function","title":"JungleSpec.TypespecGenerator.create_map_type/1","doc":"","ref":"JungleSpec.TypespecGenerator.html#create_map_type/1"},{"type":"function","title":"JungleSpec.TypespecGenerator.create_type/2","doc":"","ref":"JungleSpec.TypespecGenerator.html#create_type/2"},{"type":"macro","title":"JungleSpec.TypespecGenerator.define_object_type/4","doc":"","ref":"JungleSpec.TypespecGenerator.html#define_object_type/4"},{"type":"macro","title":"JungleSpec.TypespecGenerator.define_type/2","doc":"","ref":"JungleSpec.TypespecGenerator.html#define_type/2"},{"type":"function","title":"JungleSpec.TypespecGenerator.maybe_nullable/2","doc":"","ref":"JungleSpec.TypespecGenerator.html#maybe_nullable/2"},{"type":"function","title":"JungleSpec.TypespecGenerator.module_struct/2","doc":"","ref":"JungleSpec.TypespecGenerator.html#module_struct/2"},{"type":"function","title":"JungleSpec.TypespecGenerator.properties_types/3","doc":"","ref":"JungleSpec.TypespecGenerator.html#properties_types/3"}],"content_type":"text/markdown"}